from sendgrid_ovh.lib.OVHClientFactory import OVHClientFactory
import json


class OVHZone():

    def __init__(self):
        self.client = OVHClientFactory().build()

    def get_all_zones(self):
        return self.client.get('/domain/zone/')

    def has(self, domain):
        return domain in self.get_all_zones()

    def get_zone(self, domain):
        return self.client.get('/domain/zone/{}'.format(domain))

    def export(self, domain):
        return self.client.get('/domain/zone/{}/export'.format(domain))

    def get_records(self, domain):
        return self.client.get('/domain/zone/{}/record'.format(domain))

    def get_records_details(self, domain, sub_domain=None):
        records = []
        if sub_domain is not None:
            record_ids = self.client.get('/domain/zone/{}/record'.format(domain), subDomain=None)
        else:
            record_ids = self.client.get('/domain/zone/{}/record'.format(domain))
        for record_id in record_ids:
            records.append(self.get_record(domain, record_id))
        return records

    def get_record(self, domain, record_id):
        return self.client.get('/domain/zone/{}/record/{}'.format(domain, record_id))

    def delete_record(self, domain, record_id):
        return self.client.delete('/domain/zone/{}/record/{}'.format(domain, record_id))

    def create_record(self, domain, field_type, sub_domain, value, ttl=0):
        return self.client.post('/domain/zone/{}/record/'.format(domain),
                                    fieldType=field_type,
                                    subDomain=sub_domain,
                                    target=value,
                                    ttl=ttl)

    def update_record(self, domain, record_id, sub_domain, value, ttl=0):
        original_record = self.get_record(domain, record_id)
        return self.client.put('/domain/zone/{}/record/{}'.format(domain, record_id),
                                    subDomain=sub_domain if sub_domain is not None else original_record['subDomain'],
                                    target=value if value is not None else original_record['target'],
                                    ttl=int(ttl) if ttl is not None else original_record['ttl']
                               )

    def refresh(self, domain):
        return self.client.post('/domain/zone/{}/refresh'.format(domain))


from sendgrid_ovh.lib.OVHZone import OVHZone
from sendgrid_ovh.lib.Output import Output
import click
from rich import print
import tldextract


@click.group(name='get')
def command_get():
    pass


@click.group(name='search')
def command_search():
    pass


@click.command(name='refresh')
@click.argument('domain', required=True)
def command_refresh(domain):
    Zone().refresh(domain)


@click.command(name='list')
def command_list():
    Zone().list()


@click.command(name='has')
@click.argument('domain', required=True)
def command_has(domain):
    Zone().has(domain)


@click.command(name='export')
@click.argument('domain', required=True)
def command_export(domain):
    Zone().export(domain)


@click.command(name='records')
@click.argument('domain', required=True)
def command_get_records(domain):
    Zone().get_records(domain)


@click.command(name='zone')
@click.argument('domain', required=True)
def command_get_zone(domain):
    Zone().get_zone(domain)


@click.command(name='record')
@click.argument('domain', required=True)
@click.argument('record_id', required=True)
def command_get_record(domain, record_id):
    Zone().get_record(domain, record_id)


@click.command(name='records')
@click.argument('domain', required=True)
@click.argument('search', required=True)
def command_search_records(domain, search):
    Zone().search_records(domain, search)


@click.group(name='create')
def command_create():
    pass


@click.command(name='record')
@click.argument('full_domain', required=True)
@click.argument('value', required=True)
@click.option('--field-type', required=True, default="TXT")
@click.option('--ttl', required=False, default=0)
def command_create_record(full_domain, value, field_type, ttl):
    Zone().create_record(full_domain, value, field_type, ttl)


@click.group(name='update')
def command_update():
    pass


@click.command(name='record')
@click.argument('domain', required=True)
@click.argument('record_id', required=True)
@click.argument('value', required=False)
@click.option('--subdomain', required=False)
@click.option('--ttl', required=False, default=0)
@click.option('--refresh', required=False, default=False, is_flag=True)
def command_update_record(domain, record_id, value, subdomain, ttl, refresh):
    Zone().update_record(domain, record_id, subdomain, value, ttl, refresh)


@click.group(name='delete')
def command_delete():
    pass


@click.command(name='record')
@click.argument('domain', required=True)
@click.argument('record_id', required=True)
def command_delete_record(domain, record_id):
    Zone().delete_record(domain, record_id)


command_get.add_command(command_get_records)
command_get.add_command(command_get_record)
command_get.add_command(command_get_zone)
command_create.add_command(command_create_record)
command_search.add_command(command_search_records)
command_delete.add_command(command_delete_record)
command_update.add_command(command_update_record)


class Zone:

    def __init__(self):
        self.client = OVHZone()
        self.output = Output()
        pass

    def list(self):
        for zone in self.client.get_all_zones():
            print(zone)
        exit(0)

    def has(self, domain):
        if self.client.has(domain):
            print("[bold green][SUCCESS][/bold green] Zone [italic]{}[/italic] is manageable".format(domain))
            exit(0)
        else:
            print("[bold red][ERROR][/bold red] Zone [italic]{}[/italic] is [bold]NOT[/bold] manageable".format(domain))
            exit(1)

    def export(self, domain):
        zone = self.client.export(domain)
        print(zone)

    def get_records(self, domain):
        records = self.client.get_records_details(domain)
        self.output.ovh_zone_records(domain, records)

    def search_records(self, domain, search):
        records = self.client.get_records_details(domain, search)
        self.output.ovh_zone_records(domain, records)

    def get_zone(self, domain):
        zone = self.client.get_zone(domain)
        self.output.ovh_zone(zone)

    def get_record(self, domain, record_id):
        record = self.client.get_record(domain, record_id)
        self.output.ovh_zone_records(domain, [record])

    def create_record(self, full_domain, value, field_type, ttl=0):
        parsed_domain = tldextract.extract(full_domain)
        result = self.client.create_record(parsed_domain.registered_domain, field_type, parsed_domain.subdomain, value, ttl)
        self.output.ovh_zone_records(parsed_domain.registered_domain, [result])
        self.output.success("Record ({}) has been created".format(result['id']))

    def delete_record(self, domain, record_id):
        result = self.client.delete_record(domain, record_id)
        if result is None:
            self.output.success("Record ({}) of domain ({}) has been deleted".format(record_id, domain))

    def refresh(self, domain):
        result = self.client.refresh(domain)
        if result is None:
            self.output.success("Zone of domain ({}) has been refreshed".format(domain))

    def update_record(self, domain, record_id, sub_domain, value, ttl, refresh):
        refreshed = False
        result = self.client.update_record(domain, record_id, sub_domain, value, ttl)
        if refresh:
            refreshed = self.client.refresh(domain)
        if result is None:
            self.output.success("Record ({}) of domain ({}) has been updated".format(record_id, domain))
        if refresh and refreshed is None:
            self.output.success("Zone of domain ({}) has been refreshed".format(domain))

    def create_sendgrid_records(self, records):
        for record in records:
            self.create_record(record['host'], record['data'] + '.', record['type'].upper())







import ovh
import os
import subprocess
from sendgrid_ovh.lib.OnePasswordCli import OnePasswordCli


class OVHClientFactory:
    # https://api.ovh.com/createToken/?GET=/domain/zone/*&POST=/domain/zone/*&PUT=/domain/zone/*&DELETE=/domain/zone/*

    def __init__(self):
        self.onepassword = OnePasswordCli()

    def build(self):
        if self.onepassword.has_proper_ovh_env_variables():
            return ovh.Client(
                endpoint='ovh-eu',
                application_key=self.onepassword.get_ovh_secret('application_key'),
                application_secret=self.onepassword.get_ovh_secret('application_secret'),
                consumer_key=self.onepassword.get_ovh_secret('consumer_key'),
            )
        else:
            raise Exception('Only auth method supported is 1Password. Create the mandatory env variables')




import os
import subprocess


class OnePasswordCli:

    @staticmethod
    def has_proper_sendgrid_env_variables():
        return os.environ.get('ONEPASSWORD_SENDGRID_API_ID') is not None \
            and os.environ.get('ONEPASSWORD_SENDGRID_API_FOLDER') is not None \
            and os.environ.get('ONEPASSWORD_SENDGRID_API_ACCOUNT') is not None

    def get_sendgrid_secret(self, field):
        return self._get_secret(
            os.environ.get('ONEPASSWORD_SENDGRID_API_ACCOUNT'),
            os.environ.get('ONEPASSWORD_SENDGRID_API_FOLDER'),
            os.environ.get('ONEPASSWORD_SENDGRID_API_ID'),
            field
        )

    @staticmethod
    def has_proper_ovh_env_variables():
        return os.environ.get('ONEPASSWORD_OVH_ZONE_ID') is not None \
            and os.environ.get('ONEPASSWORD_OVH_ZONE_FOLDER') is not None \
            and os.environ.get('ONEPASSWORD_OVH_ZONE_ACCOUNT') is not None

    def _get_secret(self, account, folder, id, field ):
        code, out, error = self._op_cli("--account={} read -n op://{}/{}/{}".format(
            account,
            folder,
            id,
            field
        ))
        if code != 0:
            raise Exception('Unable to grab 1password secret field {}'.format(field))
        return out

    def get_ovh_secret(self, field):
        return self._get_secret(
            os.environ.get('ONEPASSWORD_OVH_ZONE_ACCOUNT'),
            os.environ.get('ONEPASSWORD_OVH_ZONE_FOLDER'),
            os.environ.get('ONEPASSWORD_OVH_ZONE_ID'),
            field
        )

    def _op_cli(self, cmd):
        """
        Call the 1Password cli (op)
        :param cmd: the op arguments and options
        :param inputstr: the input given to the command
        :return: A tuple of (returncode, stdout, stderr)
        """
        e = os.environ.copy()
        p = subprocess.Popen('op ' + cmd,
                             bufsize=0,
                             close_fds=True,
                             env=e,
                             shell=True,
                             stdout=subprocess.PIPE,
                             stderr=subprocess.PIPE
                             )
        try:
            out, err = self._communicate(p)
        except subprocess.CalledProcessError:
            out = ''
            err = ''
        rc = p.returncode
        return rc, out.decode('utf-8'), err.decode('utf-8')

    @staticmethod
    def _communicate(subproc, inputstr=None):
        """
        Encode the input given to the subprocess if any
        :param subproc:
        :param inputstr:
        :return: A tuple of (stdout, stderr)
        """
        if inputstr is None:
            return subproc.communicate()
        if not inputstr:
            inputstr = ''
        return subproc.communicate(inputstr.encode())




import sendgrid
import os
from sendgrid_ovh.lib.OnePasswordCli import OnePasswordCli


class SendGridClientFactory:

    def __init__(self):
        self.onepassword = OnePasswordCli()

    def build(self):
        if os.environ.get('SENDGRID_API_KEY'):
            return sendgrid.SendGridAPIClient().client
        elif self.onepassword.has_proper_sendgrid_env_variables():
            return sendgrid.SendGridAPIClient(
                api_key=self.onepassword.get_sendgrid_secret('api_key')
            ).client
        else:
            raise Exception('Unable to find Sendgrid API key in environment')



from sendgrid_ovh.lib.SendGridWhiteLabel import SendGridWhiteLabel
from sendgrid_ovh.lib.Output import Output
from sendgrid_ovh.zone.zone import Zone
import click
from rich.prompt import Confirm
from python_http_client import NotFoundError


@click.group('sendgrid')
def command_sendgrid():
    pass


@click.group('authenticated-domains')
def command_authenticated_domains():
    pass


@click.command(name='create')
@click.argument('domain', required=True)
@click.option('--zone-handler')
def command_domain_authenticate(domain, zone_handler):
    SendGrid().authenticate_a_domain(domain, zone_handler)


@click.command(name='list')
@click.option('--only-not-validated', help='Ouput only domains not validated', is_flag=True)
def command_list_all_authenticated_domains(only_not_validated):
    SendGrid().list_authenticated_domains(only_not_validated)


@click.command(name='get')
@click.argument('domain_id', required=True, type=int)
def command_get_authenticated_domain_id(domain_id):
    SendGrid().get_authenticated_domain_id(domain_id)


@click.command(name='search')
@click.argument('domain', required=True)
@click.option('--only-ids', help='Ouput only matching ids', is_flag=True)
def command_search_authenticated_domain(domain, only_ids):
    SendGrid().search_authenticated_domain(domain, only_ids)


@click.command(name='delete')
@click.argument('domain_id', required=True, type=int)
def command_delete_authenticated_domain_id(domain_id):
    SendGrid().delete_authenticated_domain_id(domain_id)


@click.command(name='validate')
@click.argument('domain_id', required=True, type=int)
def command_validate_authenticated_domain_id(domain_id):
    SendGrid().valite_authenticated_domain_id(domain_id)


command_sendgrid.add_command(command_authenticated_domains)
command_authenticated_domains.add_command(command_list_all_authenticated_domains)
command_authenticated_domains.add_command(command_domain_authenticate)
command_authenticated_domains.add_command(command_search_authenticated_domain)
command_authenticated_domains.add_command(command_get_authenticated_domain_id)
command_authenticated_domains.add_command(command_delete_authenticated_domain_id)
command_authenticated_domains.add_command(command_validate_authenticated_domain_id)


class SendGrid:

    def __init__(self):
        self.client = SendGridWhiteLabel()
        self.output = Output()
        self.ovh = Zone()

    def authenticate_a_domain(self, domain, zone_handler):
        result = self.client.create_domain_authentication(domain)
        records = result['dns']
        records['_dmarc'] = {
            'type': 'txt',
            'host': '_dmarc.' + domain,
            'data': 'v=DMARC1; p=none;',
            'valid': 'Not validated'
        }
        self.output.sendgrid_records(records.values())
        self.output.success("Domain ({}) has been created with id ({})".format(domain, result['id']))
        if zone_handler and zone_handler.lower() == 'ovh':
            self.ovh.create_sendgrid_records(records.values())
            self.ovh.refresh(domain)

    def list_authenticated_domains(self, only_not_validated):
        domains = self.client.list_all_authenticated_domains()
        if only_not_validated:
            domains = [ domain for domain in domains if not domain['valid']]
        self.output.sendgrid_domains(domains)

    def search_authenticated_domain(self, domain, only_ids):
        domains = self.client.search_authenticated_domains(domain)
        if only_ids:
            for domain in domains:
                print(domain['id'])
        else:
            self.output.sendgrid_domains(domains)

    def get_authenticated_domain_id(self, domain_id):
        domain = self.client.get_authenticated_domain_id(domain_id)
        self.output.sendgrid_records(domain['dns'].values())

    def delete_authenticated_domain_id(self, domain_id):
        if self._is_valid(domain_id):
            self.output.error("Valid domains can't be deleted by this app.Delete them manually")
            self.get_authenticated_domain_id(domain_id)
            exit(1)
        result = self.client.delete_authenticated_domain_id(domain_id)
        if result == "deleted":
            self.output.success("domain_id ({}) successfully deleted.".format(domain_id))
        else:
            self.output.error(result)
            exit(1)

    def valite_authenticated_domain_id(self, domain_id):
        result = self.client.validate_authenticated_domain_id(domain_id)
        if result['valid']:
            self.output.success("Domain_id ({}) is valid.".format(domain_id))
        else:
            self.output.error("Domain_id ({}) is NOT valid.".format(domain_id))
            self.get_authenticated_domain_id(domain_id)

    def _is_valid(self, domain_id):
        try:
            domain = self.client.get_authenticated_domain_id(domain_id)
        except NotFoundError as e:
            self.output.error("domain_id ({}) not found".format(domain_id))
            exit(2)
        return domain['valid']









from sendgrid_ovh.lib.SendGridClientFactory import SendGridClientFactory
import json


class SendGridWhiteLabel():

    def __init__(self):
        self.client = SendGridClientFactory().build()

    def create_domain_authentication(self, domain):
        data = {
            "domain": domain,
            "subdomain": "",
            "username": "",
            "ips": [],
            "custom_spf": True,
            "default": True,
            "automatic_security": True
        }
        response = self.client.whitelabel.domains.post(
            request_body=data
        )
        return json.loads(response.body.decode('utf-8'))

    def list_all_authenticated_domains(self):
        response = self.client.whitelabel.domains.get(query_params={
            'limit': 1000,
        })
        body = json.loads(response.body.decode('utf-8'))
        return body

    def search_authenticated_domains(self, domain):
        response = self.client.whitelabel.domains.get(query_params={
            'domain': domain
        })
        body = json.loads(response.body.decode('utf-8'))
        return body

    def get_authenticated_domain_id(self, domain_id):
        response = self.client.whitelabel.domains._(domain_id).get()
        body = json.loads(response.body.decode('utf-8'))
        return body

    def delete_authenticated_domain_id(self, domain_id):
        response = self.client.whitelabel.domains._(domain_id).delete()
        if response.status_code == 204:
            return "deleted"
        else:
            body = json.loads(response.body.decode('utf-8'))
            return body

    def validate_authenticated_domain_id(self, domain_id):
        response = self.client.whitelabel.domains._(domain_id).validate.post()
        body = json.loads(response.body.decode('utf-8'))
        return body




from rich.table import Table
from rich.console import Console
from rich import print
from datetime import datetime


class Output:
    @staticmethod
    def error(message):
        print("[bold red]ERROR[/bold red] {}".format(message))

    @staticmethod
    def success(message):
        print("[bold green]SUCCESS[/bold green] {}".format(message))

    @staticmethod
    def _print_table(table):
        console = Console()
        console.print(table)

    @staticmethod
    def _true_green_false_red(boolean):
        if boolean:
            return "[green]True[/green]"
        else:
            return "[red]False[/red]"

    def sendgrid_domains(self, domains, title=None):
        table = Table(title=None)
        table.add_column('id')
        table.add_column('subdomain')
        table.add_column('domain')
        table.add_column('Valid')

        for domain in domains:
            table.add_row(str(domain['id']),
                          domain['subdomain'],
                          domain['domain'],
                          self._true_green_false_red(domain['valid'])
                          )

        self._print_table(table)

    def sendgrid_records(self, records, title=None):
        table = Table(title=title)
        table.add_column('Subdomain')
        table.add_column('Type')
        table.add_column('Value')
        table.add_column('Valid')

        for record in records:
            table.add_row(record['host'],
                          record['type'],
                          record['data'],
                          self._true_green_false_red(record['valid']))

        self._print_table(table)

    def ovh_zone_records(self, domain, records):
        table = Table(title="Records of {}".format(domain))
        table.add_column('ID')
        table.add_column('Subdomain')
        table.add_column('TTL')
        table.add_column('Type')
        table.add_column('Value')

        for record in records:
            table.add_row(str(record['id']), record['subDomain'], str(record['ttl']), record['fieldType'], record['target'])

        self._print_table(table)

    def ovh_zone(self, zone):
        table = Table(title="Details of zone of {}".format(zone['name']),
                      show_header=False,
                      show_lines=True)
        table.add_column('Key')
        table.add_column('Value')

        table.add_row('dnssecSupported', str(zone['dnssecSupported']))
        table.add_row('hasDnsAnycast', str(zone['hasDnsAnycast']))
        table.add_row('lastUpdate', datetime.fromisoformat(zone['lastUpdate']).strftime("%d/%m/%Y %H:%M:%S"))
        table.add_row('nameServers', "\n".join(zone['nameServers']))

        self._print_table(table)